/*
 * BioModels Database: Web Services client library
 * Copyright (C) BioModels.net  2005-2012
 * 
 * 
 * The BioModels Database Web Services client library is free software: 
 * you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */


package uk.ac.ebi.biomodels.ws;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
//import java.util.Map;
import java.util.TreeMap;
import org.w3c.dom.*;

import javax.xml.parsers.*;
import javax.xml.rpc.ParameterMode;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.AxisProperties;

//import uk.ac.ebi.biomodels.BioModelsWebServices.modelCategory;


/**
 * Entry point of the BioModels Database Web Services client.
 * 
 * @author Camille Laibe, Chen Li
 * @version 20120416
 */
public class BioModelsWSClient
{
    // address to access to the service
    private String endpoint = "http://www.ebi.ac.uk/biomodels-main/services/BioModelsWebServices";
    // Call object represents the dynamic call of the service
    private Call call = null;
    // categories of models
    //private enum modelCategory {curated, noncurated, all};
    
    
    /**
     * Default constructor.
     */
    public BioModelsWSClient()
    {
        // nothing
    }
    
    /**
     * Sets the values of environmental variables (useful for things such as proxy, ...).
     * @param propertyName name of the property
     * @param value of the property
     */
    public void setProperty(String propertyName, String value)
    {
        AxisProperties.setProperty(propertyName, value);
    }
    
    /**
     * Sets the address of the web services endpoint.
     * @param address URL of the web services endpoint to be used
     */
    public void setEndPoint(String address)
    {
        this.endpoint = address;
    }
    
    /**
     * Retrieves the address of the web services endpoint.
     * @return address URL of the web services endpoint
     */
    public String getEndPoint()
    {
        return this.endpoint;
    }
    
    /**
     * Checks if the web services are working properly.
     * @return "Hello BioModels", if the call is a success
     * @throws BioModelsWSException
     */
    public String helloBioModels() throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("helloBioModels");
            call.setReturnType(XMLType.XSD_STRING);
            
            return (String) call.invoke(new Object[]{});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the SBML form of a model (in a string) given its identifier.
     * @param id model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return SBML model in a string, or 'null' if the provided identifier is not valid or the model does not exist
     * @throws BioModelsWSException
     * @deprecated  Replaced by {@link #getModelSBMLById(String id)}
     */
    @Deprecated
    public String getModelById(String id) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelById");
            call.addParameter("id", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return (String) call.invoke(new Object[]{id});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the SBML form of a model (in a string) given its identifier.
     * @param id model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return SBML model in a string, or 'null' if the provided identifier is not valid or the model does not exist
     * @throws BioModelsWSException 
     */
    public String getModelSBMLById(String id) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelSBMLById");
            call.addParameter("id", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return (String) call.invoke(new Object[]{id});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the name of the authors of the publication associated with a given model.
     * @param modelId model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return names of the publication's authors, or 'null' if the provided identifier is not valid or the model does not exist
     */
    public String[] getAuthorsByModelId(String modelId) throws BioModelsWSException
    {
    	try
        {
    		init();
            call.setOperationName("getAuthorsByModelId");
            call.addParameter("modelId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{modelId});
        }
    	catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the name of the encoders of a given model.
     * @param modelId model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return names of the model's encoders, or 'null' if the provided identifier is not valid or the model does not exist
     */
    public String[] getEncodersByModelId(String modelId) throws BioModelsWSException
    {
    	try
        {
    		init();
            call.setOperationName("getEncodersByModelId");
            call.addParameter("modelId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{modelId});
        }
    	catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the publication's identifier of a given model.
     * @param modelId model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return publication identifier (can be a PMID, DOI or URL), or 'null' if the provided identifier is not valid or the model does not exist
     */
    public String getPublicationByModelId(String modelId) throws BioModelsWSException
    {
    	try
        {
    		init();
            call.setOperationName("getPublicationByModelId");
            call.addParameter("modelId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return (String) call.invoke(new Object[]{modelId});
        }
    	catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the date of last modification of a given model.
     * @param modelId model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return date of last modification (expressed according to ISO 8601), or 'null' if the provided identifier is not valid or the model does not exist
     * @deprecated Use {@link #getLastModifiedDateByModelId(String)} instead.
     */
    @Deprecated
    public String getDateLastModifByModelId(String modelId) throws BioModelsWSException
    {
    	try
        {
    		init();
            call.setOperationName("getDateLastModifByModelId");
            call.addParameter("modelId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return (String) call.invoke(new Object[]{modelId});
        }
    	catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the date of last modification of a given model.
     * @param modelId model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return date of last modification (expressed according to ISO 8601), or 'null' if the provided identifier is not valid or the model does not exist
     */
    public String getLastModifiedDateByModelId(String modelId) throws BioModelsWSException
    {
    	try
        {
    		init();
            call.setOperationName("getLastModifiedDateByModelId");
            call.addParameter("modelId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return (String) call.invoke(new Object[]{modelId});
        }
    	catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    
    /**
     * Retrieves the main information (identifier, name, publication identifier and date of last modification) about a given model.
     * @param id model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return a <code>SimpleModel</code>, or 'null' if the provided identifier is not valid or the model does not exist
     * @throws BioModelsWSException 
     */
    public SimpleModel getSimpleModelById(String id) throws BioModelsWSException
    {
    	SimpleModel model = null;
    	Boolean found = false;
    	
        try
        {
            init();
            call.setOperationName("getSimpleModelById");
            call.addParameter("id", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            String responseXml = (String) call.invoke(new Object[]{id});
            
            if ((null != responseXml) && (! responseXml.isEmpty()))
    		{
	            Map<String, List<SimpleModel>> modelsMap = getSimpleModelsWithReference(responseXml);
	            if ((null != modelsMap) && (modelsMap.size() > 0))
	            {
	            	for (String key: modelsMap.keySet())
	            	{
	            		for (SimpleModel mod: modelsMap.get(key))
	            		{
	            			model = mod;
	            			found = true;
	            			break;
	            		}
	            		
	            		if (found)
	            		{
	            			break;
	            		}
	            	}
	            }
    		}
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
        
        return model;
    }
    
    /**
     * Retrieves the main information (identifier, name, publication identifier and date of last modification, ...) about given models.
     * @param id list of model identifiers (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return list of <code>SimpleModel</code>
     * @throws BioModelsWSException 
     */
    public List<SimpleModel> getSimpleModelsByIds(String[] ids) throws BioModelsWSException
    {
    	List<SimpleModel> models = null;
    	
    	try
        {
            init();
            call.setOperationName("getSimpleModelsByIds");
            call.addParameter("ids", XMLType.SOAP_ARRAY, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            String responseXml = (String) call.invoke(new Object[]{ids});
            
            if ((null != responseXml) && (! responseXml.isEmpty()))
    		{
	            models = getSimpleModels(responseXml);
    		}
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
        
        return models;
    }
    
    /**
     * Retrieves the name of a model name given its identifier.
     * @param id model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
     * @return model name, or 'null' if the provided identifier is not valid or the model does not exist
     * @throws BioModelsWSException 
     */
    public String getModelNameById(String id) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelNameById");
            call.addParameter("id", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return (String) call.invoke(new Object[]{id});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /*
    public Map<String, String> getAllModels(modelCategory category)
    {
        // TODO: complete that!
        
        do some stuff with category!!!
    	
    	<wsdl:message name="getAllModelsRequest">
    	<wsdl:part name="category" type="xsd:anyType"/>
    	</wsdl:message>
    	
        return null;
    }
    */
    
    /**
     * Retrieves the identifiers of all the published models.
     * @return list of model identifiers
     * @throws BioModelsWSException 
     */
    public String[] getAllModelsId() throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getAllModelsId");
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the identifiers of all the published curated models.
     * @return list of model identifiers
     * @throws BioModelsWSException 
     */
    public String[] getAllCuratedModelsId() throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getAllCuratedModelsId");
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the identifiers of all the published non-curated models.
     * @return list of model identifiers
     * @throws BioModelsWSException 
     */
    public String[] getAllNonCuratedModelsId() throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getAllNonCuratedModelsId");
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the models' identifiers which name includes the given keyword.
     * @param modelName part of a model name
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByName(String modelName) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByName");
            call.addParameter("modelName", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{modelName});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the identifiers of all models which have a given person as author or encoder.
     * @param personName author's or encoder's name
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByPerson(String personName) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByPerson");
            call.addParameter("personName", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{personName});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the identifiers of all models related to one (or more) publication(s).
     * @param publicationIdOrText publication identifier (PMID or DOI) or text which occurs in the publication's title or abstract
     * @return list of model identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByPublication(String publicationIdOrText) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByPublication");
            call.addParameter("publicationIdOrText", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{publicationIdOrText});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the identifiers of all the models which are annotated with a given ChEBI term.
     * @param ChEBIId identifier of a ChEBI term (e.g. CHEBI:4991)
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByChEBIId(String ChEBIId) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByChEBIId");
            call.addParameter("ChEBIId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{ChEBIId});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves all the models which are annotated with ChEBI terms.
     * @return all models annotated with ChEBI terms, as a TreeMap (which uses ChEBI identifiers as keys)
     * @throws BioModelsWSException 
     */
    public Map<String, List<SimpleModel>> getSimpleModelsRelatedWithChEBI() throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getSimpleModelsRelatedWithChEBI");
            call.setReturnType(XMLType.XSD_STRING);
            
            return getSimpleModelsWithReference((String) call.invoke(new Object[]{}));
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the models which are annotated with the given ChEBI terms.
     * @param ChEBIIds identifiers of a ChEBI terms (e.g. CHEBI:4991)
     * @return all models annotated with the provided ChEBI identifiers, as a TreeMap (which uses ChEBI identifiers as keys)
     * @throws BioModelsWSException 
     */
    public Map<String, List<SimpleModel>> getSimpleModelsByChEBIIds(String[] ChEBIIds) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getSimpleModelsByChEBIIds");
            call.addParameter("ChEBIIds", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return getSimpleModelsWithReference((String) call.invoke(new Object[]{ChEBIIds}));
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the identifiers of all models which are associated to some ChEBI terms.
     * This relies on the method 'getLiteEntity' of the ChEBI Web Services (cf. http://www.ebi.ac.uk/chebi/webServices.do).
     * @param text ChEBI identifier (e.g. CHEBI:4991) or name or synonym
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByChEBI(String text) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByChEBI");
            call.addParameter("text", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{text});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves all the models which are annotated with a given UniProt record.
     * @param UniProtId UniProt identifier (e.g. P12345)
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByUniprotId(String UniProtId) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByUniprotId");
            call.addParameter("UniProtId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{UniProtId});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves all the models which are annotated with the given UniProt records.
     * @param UniProtId list of UniProt identifiers (e.g. P12345)
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByUniprotIds(String[] UniProtIds) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByUniprotIds");
            call.addParameter("UniProtIds", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{UniProtIds});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves all the models which are annotated with the given UniProt records.
     * @param UniProtIds list of UniProt identifiers (e.g. P12345)
     * @return all the models annotated with the provided UniProt records, as a TreeMap (which uses UniProt identifiers as keys)
     * @throws BioModelsWSException 
     */
    public Map<String, List<SimpleModel>> getSimpleModelsByUniprotIds(String[] UniProtIds) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getSimpleModelsByUniprotIds");
            call.addParameter("UniProtIds", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            String xml = (String) call.invoke(new Object[]{UniProtIds});
            
            return getSimpleModelsWithReference(xml);
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves all the models which are associated to the provided UniProt text.
     * @param text free (UniProt based) text
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByUniprot(String text) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByUniprot");
            call.addParameter("text", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{text});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves all the models which are annotated with the given Reactome records.
     * @param ReactomeIds list of Reactome identifiers (e.g. REACT_1590)
     * @return models annotated with the provided Reactome identifiers, as a TreeMap (which uses Reactome identifiers as keys)
     * @throws BioModelsWSException 
     */
    public Map<String, List<SimpleModel>> getSimpleModelsByReactomeIds(String[] ReactomeIds) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getSimpleModelsByReactomeIds");
            call.addParameter("ReactomeIds", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return getSimpleModelsWithReference((String) call.invoke(new Object[]{ReactomeIds}));
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the models which are annotated with the given Gene Ontology term.
     * @param GOId Gene Ontology identifier (e.g. GO:0006915)
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByGOId(String GOId) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByGOId");
            call.addParameter("GOId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{GOId});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the models which are associated to the provided Gene Ontology text.
     * @param text free (GO based) text
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByGO(String text) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByGO");
            call.addParameter("text", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{text});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the models which are annotated with the given taxon.
     * @param taxonomyId Taxonomy identifier (e.g. 9606)
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByTaxonomyId(String taxonomyId) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByTaxonomyId");
            call.addParameter("taxonomyId", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{taxonomyId});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Retrieves the models which are associated to the provided Taxonomy text.
     * @param text free (Taxonomy based) text
     * @return list of models identifiers
     * @throws BioModelsWSException 
     */
    public String[] getModelsIdByTaxonomy(String text) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getModelsIdByTaxonomy");
            call.addParameter("text", XMLType.XSD_STRING, ParameterMode.IN);
            call.setReturnType(XMLType.SOAP_ARRAY);
            
            return (String[]) call.invoke(new Object[]{text});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    /**
     * Generates the minimal sub-model of a given model in the database including all selected components.
     * @param modelId identifier of the model from which the sub-model will be extracted
     * @param elementsIds identifiers of the selected elements. Currently only supports identifiers from compartments, species, and reactions.
     * @return the sub-model encoded in SBML
     * @throws BioModelsWSException 
     */
    public String getSubModelSBML(String modelId, String[] elementsIds) throws BioModelsWSException
    {
        try
        {
            init();
            call.setOperationName("getSubModelSBML");
            call.addParameter("modelId", XMLType.XSD_STRING, ParameterMode.IN);
            call.addParameter("elementsIds", XMLType.SOAP_ARRAY, ParameterMode.IN);
            call.setReturnType(XMLType.XSD_STRING);
            
            return (String) call.invoke(new Object[]{modelId, elementsIds});
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    
    /*
     * ###################################### private methods ###########################
     */
    
    
    /*
     * initialisation
     */
    private void init() throws BioModelsWSException
    {
        try
        {
            Service service = new Service();
            call = (Call) service.createCall();
            call.setTargetEndpointAddress(new java.net.URL(endpoint));
        }
        catch (Exception e)
        {
            throw new BioModelsWSException(e);
        }
    }
    
    
    /*
     * parses an XML stream and retrieve the model(s), then puts them in a list.
     */
    private List<SimpleModel> getSimpleModels(String xml) throws BioModelsWSException
    {
    	List<SimpleModel> models = null;
    	
    	if (null != xml)
    	{
    		models = new ArrayList<SimpleModel>();
    		
 	        try
 	        {
 	            ByteArrayInputStream is = new ByteArrayInputStream(xml.getBytes());
 	            // creates a factory
 	            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
 	            // uses the factory to create a builder
 	            DocumentBuilder builder = factory.newDocumentBuilder();
 	            Document doc = builder.parse(is);
 	            // gets a list of all elements in the document
 	            NodeList list = doc.getElementsByTagName("simpleModel");
 	            
 	            for (int i=0; i<list.getLength(); i++)
	            {
	            	Element element = (Element) list.item(i);
	            	SimpleModel model = getSimpleModelFromXML(element);
	            	models.add(model);
	            }
 	        }
 	        catch (Exception e)
	        {
	            throw new BioModelsWSException(e);
	        }
    	}
    	
    	return models;
    }
    
    /*
     * parses an XML stream and retrieve the model(s), then puts them into a TreeMap with the publication identifier as key value.
     */
    private Map<String, List<SimpleModel>> getSimpleModelsWithReference(String xml) throws BioModelsWSException
    {
        Map<String, List<SimpleModel>> modelsMap = null;
        
        if (null != xml)
        {
        	modelsMap = new TreeMap<String, List<SimpleModel>>();
	        
	        try
	        {
	            ByteArrayInputStream is = new ByteArrayInputStream(xml.getBytes());
	            // creates a factory
	            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	            // uses the factory to create a builder
	            DocumentBuilder builder = factory.newDocumentBuilder();
	            Document doc = builder.parse(is);
	            // gets a list of all elements in the document
	            NodeList list = doc.getElementsByTagName("simpleModel");
	            
	            for (int i=0; i<list.getLength(); i++)
	            {
	            	Element element = (Element) list.item(i);
	            	SimpleModel model = getSimpleModelFromXML(element);
	            	String referenceId = element.getElementsByTagName("referenceId").item(0).getTextContent();
	            	
	                if (modelsMap.containsKey(referenceId))
	                {
	                    (modelsMap.get(referenceId)).add(model);
	                }
	                else
	                {
	                    ArrayList<SimpleModel> modelsList = new ArrayList<SimpleModel>();
	                    modelsList.add(model);
	                    modelsMap.put(referenceId, modelsList);
	                }
	            }
	        }
	        catch (Exception e)
	        {
	            throw new BioModelsWSException(e);
	        }
        }
        
        return modelsMap;
    }
    
    /*
     * Retrieves one SimpleModel from a partial piece of XML.
     */
	private SimpleModel getSimpleModelFromXML(Element element)
	{
		SimpleModel model = null;
		
        List<String> encoders = new ArrayList<String>();
        NodeList parentElts = element.getElementsByTagName("encoders");
        for (int j=0; j<parentElts.getLength(); j++)
        {
        	Element elt = (Element) parentElts.item(j);
        	NodeList childrenElts = elt.getElementsByTagName("encoder");
        	for (int k=0; k<childrenElts.getLength(); ++k)
        	{
        		encoders.add(childrenElts.item(k).getTextContent());
        	}
        }
        List<String> authors = new ArrayList<String>();
        parentElts = element.getElementsByTagName("authors");
        for (int j=0; j<parentElts.getLength(); j++)
        {
        	Element elt = (Element) parentElts.item(j);
        	NodeList childrenElts = elt.getElementsByTagName("author");
        	for (int k=0; k<childrenElts.getLength(); ++k)
        	{
        		authors.add(childrenElts.item(k).getTextContent());
        	}
        }
        model = new SimpleModel(
                element.getElementsByTagName("modelId").item(0).getTextContent(),
                element.getElementsByTagName("modelSubmissionId").item(0).getTextContent(),
                element.getElementsByTagName("modelName").item(0).getTextContent(),
                encoders,
                element.getElementsByTagName("publicationId").item(0).getTextContent(),
                authors,
                element.getElementsByTagName("lastModificationDate").item(0).getTextContent());
		
		return model;
	}
}
